@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012 Red Hat, Inc.
+ * Copyright (c) 2011, 2012, 2014 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -73,12 +73,13 @@ let settings = null;
 
 // used by the application, but not by the search provider
 let changeMonitor = null;
-let collectionManager = null;
 let cssProvider = null;
 let documentManager = null;
 let modeController = null;
 let notificationManager = null;
-let offsetController = null;
+let offsetCollectionsController = null;
+let offsetDocumentsController = null;
+let offsetSearchController = null;
 let queryBuilder = null;
 let searchCategoryManager = null;
 let searchController = null;
@@ -86,7 +87,9 @@ let searchMatchManager = null;
 let searchTypeManager = null;
 let selectionController = null;
 let sourceManager = null;
-let trackerController = null;
+let trackerCollectionsController = null;
+let trackerDocumentsController = null;
+let trackerSearchController = null;
 
 const TrackerExtractPriorityIface = '<node> \
 <interface name="org.freedesktop.Tracker1.Extract.Priority"> \
@@ -315,6 +318,20 @@ const Application = new Lang.Class({
                             let action = this.lookup_action(actionEntry.name);
                             action.set_enabled(mode == actionEntry.window_mode);
                         }));
+                } else if (actionEntry.window_modes) {
+                    modeController.connect('window-mode-changed', Lang.bind(this,
+                        function() {
+                            let mode = modeController.getWindowMode();
+                            let action = this.lookup_action(actionEntry.name);
+                            let enable = false;
+                            for (let idx in actionEntry.window_modes) {
+                                if (mode == actionEntry.window_modes[idx]) {
+                                    enable = true;
+                                    break;
+                                }
+                            }
+                            action.set_enabled(enable);
+                        }));
                 }
             }));
     },
@@ -442,9 +459,9 @@ const Application = new Lang.Class({
 
         application = this;
         if (!application.isBooks)
-            settings = new Gio.Settings({ schema: 'org.gnome.documents' });
+            settings = new Gio.Settings({ schema_id: 'org.gnome.documents' });
         else
-            settings = new Gio.Settings({ schema: 'org.gnome.books' });
+            settings = new Gio.Settings({ schema_id: 'org.gnome.books' });
 
         let gtkSettings = Gtk.Settings.get_default();
         gtkSettings.connect('notify::gtk-theme-name', Lang.bind(this, this._themeChanged));
@@ -458,14 +475,6 @@ const Application = new Lang.Class({
             return;
         }
 
-        try {
-            this._extractPriority = TrackerExtractPriority();
-            this._extractPriority.SetRdfTypesRemote(['nfo:Document']);
-        } catch (e) {
-            log('Unable to connect to the tracker extractor: ' + e.toString());
-            return;
-        }
-
         if (!application.isBooks) {
             try {
                 goaClient = Goa.Client.new_sync(null);
@@ -476,15 +485,19 @@ const Application = new Lang.Class({
         }
 
         connectionQueue = new TrackerController.TrackerConnectionQueue();
+        changeMonitor = new ChangeMonitor.TrackerChangeMonitor();
 
         // now init application components
         Search.initSearch(imports.application);
         Search.initSearch(imports.shellSearchProvider);
 
-        changeMonitor = new ChangeMonitor.TrackerChangeMonitor();
-        documentManager = new Documents.DocumentManager();
         modeController = new WindowMode.ModeController();
-        trackerController = new TrackerController.TrackerController();
+        offsetCollectionsController = new Search.OffsetCollectionsController();
+        offsetDocumentsController = new Search.OffsetDocumentsController();
+        offsetSearchController = new Search.OffsetSearchController();
+        trackerCollectionsController = new TrackerController.TrackerCollectionsController();
+        trackerDocumentsController = new TrackerController.TrackerDocumentsController();
+        trackerSearchController = new TrackerController.TrackerSearchController();
         selectionController = new Selections.SelectionController();
 
         this._actionEntries = [
@@ -514,7 +527,9 @@ const Application = new Lang.Class({
               create_hook: this._viewAsCreateHook,
               parameter_type: 's',
               state: settings.get_value('view-as'),
-              window_mode: WindowMode.WindowMode.OVERVIEW },
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] },
             { name: 'open-current',
               callback: this._onActionOpenCurrent,
               window_mode: WindowMode.WindowMode.PREVIEW },
@@ -546,9 +561,13 @@ const Application = new Lang.Class({
             { name: 'rotate-right', accel: '<Primary>Right',
               window_mode: WindowMode.WindowMode.PREVIEW },
             { name: 'select-all', accel: '<Primary>a',
-              window_mode: WindowMode.WindowMode.OVERVIEW },
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] },
             { name: 'select-none',
-              window_mode: WindowMode.WindowMode.OVERVIEW },
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] },
             { name: 'properties',
               callback: this._onActionProperties,
               window_mode: WindowMode.WindowMode.PREVIEW },
@@ -566,15 +585,21 @@ const Application = new Lang.Class({
             { name: 'search-source',
               parameter_type: 's',
               state: GLib.Variant.new('s', Search.SearchSourceStock.ALL),
-              window_mode: WindowMode.WindowMode.OVERVIEW },
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] },
             { name: 'search-type',
               parameter_type: 's',
               state: GLib.Variant.new('s', Search.SearchTypeStock.ALL),
-              window_mode: WindowMode.WindowMode.OVERVIEW },
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] },
             { name: 'search-match',
               parameter_type: 's',
               state: GLib.Variant.new('s', Search.SearchMatchStock.ALL),
-              window_mode: WindowMode.WindowMode.OVERVIEW }
+              window_modes: [WindowMode.WindowMode.COLLECTIONS,
+                             WindowMode.WindowMode.DOCUMENTS,
+                             WindowMode.WindowMode.SEARCH] }
         ];
 
         this.gdataMiner = new Miners.GDataMiner();
@@ -588,11 +613,6 @@ const Application = new Lang.Class({
             this._initGettingStarted();
     },
 
-    vfunc_shutdown: function() {
-        this._extractPriority.ClearRdfTypesRemote();
-        this.parent();
-    },
-
     _createWindow: function() {
         if (this._mainWindow)
             return;
@@ -602,6 +622,13 @@ const Application = new Lang.Class({
         this._mainWindow = new MainWindow.MainWindow(this);
         this._mainWindow.window.connect('destroy', Lang.bind(this, this._onWindowDestroy));
 
+        try {
+            this._extractPriority = TrackerExtractPriority();
+            this._extractPriority.SetRdfTypesRemote(['nfo:Document']);
+        } catch (e) {
+            log('Unable to connect to the tracker extractor: ' + e.toString());
+        }
+
         // start miners
         this._startMiners();
     },
@@ -622,7 +649,7 @@ const Application = new Lang.Class({
     vfunc_activate: function() {
         if (!this._mainWindow) {
             this._createWindow();
-            modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
+            modeController.setWindowMode(WindowMode.WindowMode.DOCUMENTS);
         }
 
         this._mainWindow.window.present_with_time(this._activationTimestamp);
@@ -633,7 +660,12 @@ const Application = new Lang.Class({
         // clean up signals
         changeMonitor.disconnectAll();
         documentManager.disconnectAll();
-        trackerController.disconnectAll();
+        offsetCollectionsController.disconnectAll();
+        offsetDocumentsController.disconnectAll();
+        offsetSearchController.disconnectAll();
+        trackerCollectionsController.disconnectAll();
+        trackerDocumentsController.disconnectAll();
+        trackerSearchController.disconnectAll();
         selectionController.disconnectAll();
         modeController.disconnectAll();
         this.disconnectAllJS();
@@ -646,6 +678,9 @@ const Application = new Lang.Class({
 
         // stop miners
         this._stopMiners();
+
+        if (this._extractPriority)
+            this._extractPriority.ClearRdfTypesRemote();
     },
 
     _onWindowDestroy: function(window) {
@@ -683,7 +718,8 @@ const Application = new Lang.Class({
             // forward the search terms next time we enter the overview
             let modeChangeId = modeController.connect('window-mode-changed', Lang.bind(this,
                 function(object, newMode) {
-                    if (newMode != WindowMode.WindowMode.OVERVIEW)
+                    if (newMode == WindowMode.WindowMode.EDIT
+                        || newMode == WindowMode.WindowMode.PREVIEW)
                         return;
 
                     modeController.disconnect(modeChangeId);
@@ -696,7 +732,7 @@ const Application = new Lang.Class({
 
     _onLaunchSearch: function(provider, terms, timestamp) {
         this._createWindow();
-        modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
+        modeController.setWindowMode(WindowMode.WindowMode.DOCUMENTS);
         searchController.setString(terms.join(' '));
         this.change_action_state('search', GLib.Variant.new('b', true));
 
