@@ -94,18 +94,27 @@ const OverviewToolbar = new Lang.Class({
     Name: 'OverviewToolbar',
     Extends: MainToolbar,
 
-    _init: function(overlay) {
+    _init: function(overlay, stack) {
         this._overlay = overlay;
         this._collBackButton = null;
         this._collectionId = 0;
         this._selectionChangedId = 0;
-        this._selectionMenu = null;
         this._viewGridButton = null;
         this._viewListButton = null;
         this._viewSettingsId = 0;
 
         this.parent();
 
+        let builder = new Gtk.Builder();
+        builder.add_from_resource('/org/gnome/documents/selection-menu.ui');
+        let selectionMenu = builder.get_object('selection-menu');
+        this._selectionMenu = new Gtk.MenuButton({ menu_model: selectionMenu });
+        this._selectionMenu.get_style_context().add_class('selection-menu');
+
+        this._stackSwitcher = new Gtk.StackSwitcher({ no_show_all: true,
+                                                      stack: stack });
+        this._stackSwitcher.show();
+
         // setup listeners to mode changes that affect the toolbar layout
         this._selectionModeId = Application.selectionController.connect('selection-mode-changed',
             Lang.bind(this, this._resetToolbarMode));
@@ -154,7 +163,7 @@ const OverviewToolbar = new Lang.Class({
 
     _setToolbarTitle: function() {
         let selectionMode = Application.selectionController.getSelectionMode();
-        let activeCollection = Application.collectionManager.getActiveItem();
+        let activeCollection = Application.documentManager.getActiveCollection();
         let primary = null;
 
         if (!selectionMode) {
@@ -177,7 +186,7 @@ const OverviewToolbar = new Lang.Class({
                 primary = label;
         }
 
-        if (this._selectionMenu) {
+        if (selectionMode) {
             if (primary) {
                 this._selectionMenu.set_label(primary);
                 this._selectionMenu.get_child().use_markup = true;
@@ -189,12 +198,6 @@ const OverviewToolbar = new Lang.Class({
 
     _populateForSelectionMode: function() {
         this.toolbar.get_style_context().add_class('selection-mode');
-
-        let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/documents/selection-menu.ui');
-        let selectionMenu = builder.get_object('selection-menu');
-        this._selectionMenu = new Gtk.MenuButton({ menu_model: selectionMenu });
-        this._selectionMenu.get_style_context().add_class('selection-menu');
         this.toolbar.set_custom_title(this._selectionMenu);
 
         let selectionButton = new Gtk.Button({ label: _("Cancel") });
@@ -213,7 +216,7 @@ const OverviewToolbar = new Lang.Class({
     },
 
     _checkCollectionBackButton: function() {
-        let item = Application.collectionManager.getActiveItem();
+        let item = Application.documentManager.getActiveCollection();
 
         if (item && !this._collBackButton) {
             this._collBackButton = this.addBackButton();
@@ -236,6 +239,7 @@ const OverviewToolbar = new Lang.Class({
 
     _populateForOverview: function() {
         this.toolbar.set_show_close_button(true);
+        this.toolbar.set_custom_title(this._stackSwitcher);
         this._checkCollectionBackButton();
 
         let selectionButton = new Gtk.Button({ image: new Gtk.Image ({ icon_name: 'object-select-symbolic' }),
@@ -251,19 +255,18 @@ const OverviewToolbar = new Lang.Class({
 
         // connect to active collection changes while in this mode
         this._collectionId =
-            Application.collectionManager.connect('active-changed',
+            Application.documentManager.connect('active-collection-changed',
                                              Lang.bind(this, this._onActiveCollectionChanged));
     },
 
     _clearStateData: function() {
         this._collBackButton = null;
-        this._selectionMenu = null;
         this._viewGridButton = null;
         this._viewListButton = null;
         this.toolbar.set_custom_title(null);
 
         if (this._collectionId != 0) {
-            Application.collectionManager.disconnect(this._collectionId);
+            Application.documentManager.disconnect(this._collectionId);
             this._collectionId = 0;
         }
 
