@@ -149,16 +149,27 @@ const MainWindow = new Lang.Class({
 
     _goBack: function() {
         let windowMode = Application.modeController.getWindowMode();
-        let activeCollection = Application.collectionManager.getActiveItem();
+        let activeCollection = Application.documentManager.getActiveCollection();
         let handled = true;
 
-        if (windowMode == WindowMode.WindowMode.PREVIEW ||
-            windowMode == WindowMode.WindowMode.EDIT) {
-            Application.documentManager.setActiveItem(null);
-        } else if (windowMode == WindowMode.WindowMode.OVERVIEW && activeCollection) {
-            Application.documentManager.activatePreviousCollection();
-        } else {
+        switch (windowMode) {
+        case WindowMode.WindowMode.NONE:
+        case WindowMode.WindowMode.DOCUMENTS:
             handled = false;
+            break;
+        case WindowMode.WindowMode.EDIT:
+        case WindowMode.WindowMode.PREVIEW:
+            Application.documentManager.setActiveItem(null);
+            Application.modeController.goBack();
+            break;
+        case WindowMode.WindowMode.COLLECTIONS:
+        case WindowMode.WindowMode.SEARCH:
+            if (activeCollection)
+                Application.documentManager.activatePreviousCollection();
+            break;
+        default:
+            throw(new Error('Not handled'));
+            break;
         }
 
         return handled;
@@ -191,7 +202,9 @@ const MainWindow = new Lang.Class({
             return false;
         case WindowMode.WindowMode.PREVIEW:
             return this._handleKeyPreview(event);
-        case WindowMode.WindowMode.OVERVIEW:
+        case WindowMode.WindowMode.COLLECTIONS:
+        case WindowMode.WindowMode.DOCUMENTS:
+        case WindowMode.WindowMode.SEARCH:
             return this._handleKeyOverview(event);
         case WindowMode.WindowMode.EDIT:
             return false;
@@ -234,10 +247,12 @@ const MainWindow = new Lang.Class({
         if (keyval == Gdk.KEY_Escape) {
             let model = preview.getModel();
 
-            if (preview.controlsVisible && (model != null))
+            if (preview.controlsVisible && (model != null)) {
                 preview.controlsVisible = false;
-            else if (fullscreen)
+            } else if (fullscreen) {
                 Application.documentManager.setActiveItem(null);
+                Application.modeController.goBack();
+            }
 
             return false;
         }
