@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012 Red Hat, Inc.
+ * Copyright (c) 2011, 2012, 2014 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -20,6 +20,7 @@
  */
 
 const Application = imports.application;
+const Documents = imports.documents;
 const Manager = imports.manager;
 const Query = imports.query;
 
@@ -34,13 +35,12 @@ const _ = imports.gettext.gettext;
 const C_ = imports.gettext.pgettext;
 
 function initSearch(context) {
-    context.collectionManager = new Manager.BaseManager(context);
+    context.documentManager = new Documents.DocumentManager();
     context.sourceManager = new SourceManager(context);
     context.searchCategoryManager = new SearchCategoryManager(context);
     context.searchMatchManager = new SearchMatchManager(context);
     context.searchTypeManager = new SearchTypeManager(context);
     context.searchController = new SearchController(context);
-    context.offsetController = new OffsetController(context);
     context.queryBuilder = new Query.QueryBuilder(context);
 };
 
@@ -137,6 +137,12 @@ const SearchCategoryManager = new Lang.Class({
         // this._categories[category.id] = category;
 
         this.setActiveItem(recent);
+    },
+
+    getFilter: function(flags) {
+        // Since we don't expose the SearchCategoryManager in the UI,
+        // this is a placeholder for the moment.
+        return '(true)';
     }
 });
 
@@ -233,6 +239,17 @@ const SearchTypeManager = new Lang.Class({
         return [ activeItem ];
     },
 
+    getDocumentTypes: function() {
+        let types = [];
+
+        types.push(this.getItemById(SearchTypeStock.PDF));
+        types.push(this.getItemById(SearchTypeStock.PRESENTATIONS));
+        types.push(this.getItemById(SearchTypeStock.SPREADSHEETS));
+        types.push(this.getItemById(SearchTypeStock.TEXTDOCS));
+
+        return types;
+    },
+
     getAllTypes: function() {
         let types = [];
 
@@ -308,7 +325,10 @@ const SearchMatchManager = new Lang.Class({
         this.setActiveItemById(SearchMatchStock.ALL);
     },
 
-    getFilter: function() {
+    getFilter: function(flags) {
+        if ((flags & Query.QueryFlags.SEARCH) == 0)
+            return '(true)';
+
         let terms = this.context.searchController.getTerms();
         let filters = [];
 
@@ -316,7 +336,16 @@ const SearchMatchManager = new Lang.Class({
             this.forEachItem(function(item) {
                 item.setFilterTerm(terms[i]);
             });
-            filters.push(this.parent());
+
+            let filter;
+            let item = this.getActiveItem();
+
+            if (item.id == SearchMatchStock.ALL)
+                filter = this.getAllFilter();
+            else
+                filter = item.getFilter();
+
+            filters.push(filter);
         }
         return filters.length ? '( ' + filters.join(' && ') + ')' : '(true)';
     }
@@ -495,6 +524,24 @@ const SourceManager = new Lang.Class({
         this.processNewItems(newItems);
     },
 
+    getFilter: function(flags) {
+        let item;
+
+        if (flags & Query.QueryFlags.SEARCH)
+            item = this.getActiveItem();
+        else
+            item = this.getItemById(SearchSourceStock.ALL);
+
+        let filter;
+
+        if (item.id == SearchSourceStock.ALL)
+            filter = this.getAllFilter();
+        else
+            filter = item.getFilter();
+
+        return filter;
+    },
+
     getFilterNotLocal: function() {
         let sources = this.getItems();
         let filters = [];
@@ -543,26 +590,25 @@ const SourceManager = new Lang.Class({
     }
 });
 
-const _OFFSET_STEP = 50;
+const OFFSET_STEP = 50;
 
 const OffsetController = new Lang.Class({
     Name: 'OffsetController',
 
-    _init: function(context) {
+    _init: function() {
         this._offset = 0;
         this._itemCount = 0;
-        this._context = context;
     },
 
     // to be called by the view
     increaseOffset: function() {
-        this._offset += _OFFSET_STEP;
+        this._offset += OFFSET_STEP;
         this.emit('offset-changed', this._offset);
     },
 
     // to be called by the model
     resetItemCount: function() {
-        let query = this._context.queryBuilder.buildCountQuery();
+        let query = this.getQuery();
 
         Application.connectionQueue.add
             (query.sparql, null, Lang.bind(this,
@@ -589,6 +635,10 @@ const OffsetController = new Lang.Class({
                 }));
     },
 
+    getQuery: function() {
+        log('Error: OffsetController implementations must override getQuery');
+    },
+
     // to be called by the model
     resetOffset: function() {
         this._offset = 0;
@@ -599,11 +649,11 @@ const OffsetController = new Lang.Class({
     },
 
     getRemainingDocs: function() {
-        return (this._itemCount - (this._offset + _OFFSET_STEP));
+        return (this._itemCount - (this._offset + OFFSET_STEP));
     },
 
     getOffsetStep: function() {
-        return _OFFSET_STEP;
+        return OFFSET_STEP;
     },
 
     getOffset: function() {
@@ -611,3 +661,50 @@ const OffsetController = new Lang.Class({
     }
 });
 Signals.addSignalMethods(OffsetController.prototype);
+
+const OffsetCollectionsController = new Lang.Class({
+    Name: 'OffsetCollectionsController',
+    Extends: OffsetController,
+
+    _init: function() {
+        this.parent();
+    },
+
+    getQuery: function() {
+        let activeCollection = Application.documentManager.getActiveCollection();
+        let flags;
+
+        if (activeCollection)
+            flags = Query.QueryFlags.NONE;
+        else
+            flags = Query.QueryFlags.COLLECTIONS;
+
+        return Application.queryBuilder.buildCountQuery(flags);
+    }
+});
+
+const OffsetDocumentsController = new Lang.Class({
+    Name: 'OffsetDocumentsController',
+    Extends: OffsetController,
+
+    _init: function() {
+        this.parent();
+    },
+
+    getQuery: function() {
+        return Application.queryBuilder.buildCountQuery(Query.QueryFlags.DOCUMENTS);
+    }
+});
+
+const OffsetSearchController = new Lang.Class({
+    Name: 'OffsetSearchController',
+    Extends: OffsetController,
+
+    _init: function() {
+        this.parent();
+    },
+
+    getQuery: function() {
+        return Application.queryBuilder.buildCountQuery(Query.QueryFlags.SEARCH);
+    }
+});
