@@ -28,9 +28,11 @@ const Application = imports.application;
 
 const WindowMode = {
     NONE: 0,
-    OVERVIEW: 1,
+    DOCUMENTS: 1,
     PREVIEW: 2,
-    EDIT: 3
+    EDIT: 3,
+    COLLECTIONS: 4,
+    SEARCH: 5
 };
 
 const ModeController = new Lang.Class({
@@ -39,7 +41,28 @@ const ModeController = new Lang.Class({
     _init: function() {
         this._mode = WindowMode.NONE;
         this._fullscreen = false;
-        this._canFullscreen = false;
+        this._history = [];
+    },
+
+    goBack: function(steps) {
+        if (!steps)
+            steps = 1;
+
+        let oldMode;
+
+        for (let i = 0; i < steps; i++) {
+            oldMode = this._history.pop();
+            if (!oldMode || oldMode == WindowMode.NONE)
+                return;
+        }
+
+        // Swap the old and current modes.
+        let tmp = oldMode;
+        oldMode = this._mode;
+        this._mode = tmp;
+
+        this._updateFullscreen();
+        this.emit('window-mode-changed', this._mode, oldMode);
     },
 
     setWindowMode: function(mode) {
@@ -48,15 +71,10 @@ const ModeController = new Lang.Class({
         if (oldMode == mode)
             return;
 
-        if (mode == WindowMode.PREVIEW
-            || mode == WindowMode.EDIT) {
-            this.setCanFullscreen(true);
-        } else {
-            this.setCanFullscreen(false);
-        }
-
+        this._history.push(oldMode);
         this._mode = mode;
 
+        this._updateFullscreen();
         this.emit('window-mode-changed', this._mode, oldMode);
     },
 
@@ -64,11 +82,9 @@ const ModeController = new Lang.Class({
         return this._mode;
     },
 
-    setCanFullscreen: function(canFullscreen) {
-        this._canFullscreen = canFullscreen;
-
-        if (!this._canFullscreen && this._fullscreen)
-            this.setFullscreen(false);
+    _updateFullscreen: function() {
+        if (!this.getCanFullscreen() && this._fullscreen)
+            this._setFullscreen(false);
 
         this.emit('can-fullscreen-changed');
     },
@@ -90,7 +106,7 @@ const ModeController = new Lang.Class({
     },
 
     getCanFullscreen: function() {
-        return this._canFullscreen;
+        return (this._mode == WindowMode.PREVIEW || this._mode == WindowMode.EDIT);
     }
 });
 Signals.addSignalMethods(ModeController.prototype);
